'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

var EventCallableRegsitry = /*#__PURE__*/function () {
  function EventCallableRegsitry() {
    this.registry = new Map();
  }

  var _proto = EventCallableRegsitry.prototype;

  _proto.addEventListener = function addEventListener(name, callback) {
    var callbacks = this.registry.get(name);

    if (callbacks) {
      callbacks.push(callback);
    } else {
      this.registry.set(name, [callback]);
    }
  };

  _proto.removeEventListener = function removeEventListener(name, callback) {
    var callbacks = this.registry.get(name);

    if (callbacks) {
      var indexOf = callbacks.indexOf(callback);

      if (indexOf > -1) {
        callbacks.splice(indexOf, 1);
      }
    }
  };

  _proto.dispatchEvent = function dispatchEvent(event) {
    // @ts-ignore
    if (window['debug']) {
      console.log('DEBUG::', 'Event:', event.name);
    }

    var callbacks = this.registry.get(event.name);

    for (var i in callbacks) {
      callbacks[i](event);
      if (!event.propagate) break;
    }
  };

  return EventCallableRegsitry;
}();

var PluginStore = /*#__PURE__*/function () {
  function PluginStore() {
    this._eventCallableRegistry = new EventCallableRegsitry();
    this.functionArray = new Map();
    this.pluginMap = new Map();
  }

  var _proto = PluginStore.prototype;

  _proto.install = function install(key, plugin) {
    this.pluginMap.set(key, plugin);
    plugin.init(this);
    plugin.activate();
  };

  _proto.addFunction = function addFunction(key, fn) {
    this.functionArray.set(key, fn);
  };

  _proto.executeFunction = function executeFunction(key) {
    var fn = this.functionArray.get(key);

    if (fn) {
      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      return fn.apply(void 0, args);
    }

    console.error('No function added for the key ' + key + '.');
  };

  _proto.removeFunction = function removeFunction(key) {
    this.functionArray["delete"](key);
  };

  _proto.uninstall = function uninstall(key) {
    var plugin = this.pluginMap.get(key);

    if (plugin) {
      plugin.deactivate();
      this.pluginMap["delete"](key);
    }
  };

  _proto.addEventListener = function addEventListener(name, callback) {
    this._eventCallableRegistry.addEventListener(name, callback);
  };

  _proto.removeEventListener = function removeEventListener(name, callback) {
    this._eventCallableRegistry.removeEventListener(name, callback);
  };

  _proto.dispatchEvent = function dispatchEvent(event) {
    this._eventCallableRegistry.dispatchEvent(event);
  };

  return PluginStore;
}();

function createPluginStore() {
  return new PluginStore();
}

var PluginStoreContext = /*#__PURE__*/React__default.createContext( /*#__PURE__*/new PluginStore());

var PluginProvider = function PluginProvider(_ref) {
  var pluginStore = _ref.pluginStore,
      children = _ref.children;
  return React__default.createElement(PluginStoreContext.Provider, {
    value: pluginStore
  }, children);
};

function usePluginStore() {
  return React.useContext(PluginStoreContext);
}

var Renderer = function Renderer(_ref) {
  var placement = _ref.placement;
  var pluginStore = usePluginStore();
  var components = pluginStore.executeFunction('RendererPlugin.getComponentsInPosition', placement);
  return React__default.createElement(React__default.Fragment, null, components.map(function (Component) {
    return React__default.createElement(Component, null);
  }));
};

var RendererPlugin = /*#__PURE__*/function () {
  function RendererPlugin() {
    this.pluginStore = new PluginStore();
    this.componentMap = new Map();
  }

  var _proto = RendererPlugin.prototype;

  _proto.init = function init(pluginStore) {
    this.pluginStore = pluginStore;
  };

  _proto.addToComponentMap = function addToComponentMap(position, component) {
    var array = this.componentMap.get(position);

    if (!array) {
      array = [component];
    }

    this.componentMap.set(position, array);
  };

  _proto.getRendererComponent = function getRendererComponent() {
    return Renderer;
  };

  _proto.getComponentsInPosition = function getComponentsInPosition(position) {
    var componentArray = this.componentMap.get(position);
    if (!componentArray) return [];
    return componentArray;
  };

  _proto.activate = function activate() {
    this.pluginStore.addFunction('RendererPlugin.add', this.addToComponentMap.bind(this));
    this.pluginStore.addFunction('RendererPlugin.getComponentsInPosition', this.getComponentsInPosition.bind(this));
    this.pluginStore.addFunction('RendererPlugin.getRendererComponent', this.getRendererComponent.bind(this));
  };

  _proto.deactivate = function deactivate() {
    this.pluginStore.removeFunction('RendererPlugin.add');
    this.pluginStore.removeFunction('RendererPlugin.getComponentsInPosition');
    this.pluginStore.removeFunction('RendererPlugin.getRendererComponent');
  };

  return RendererPlugin;
}();

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var Event = /*#__PURE__*/function () {
  function Event(name) {
    this.name = name; // this.target = target;
    // this.currentTarget = currentTarget;

    this._propagate = true;
    this._defaults = true;
  }

  var _proto = Event.prototype;

  _proto.stopPropagation = function stopPropagation() {
    this._propagate = false;
  };

  _proto.preventDefault = function preventDefault() {
    this._defaults = false;
  };

  _createClass(Event, [{
    key: "propagate",
    get: function get() {
      return this._propagate;
    }
  }, {
    key: "defaults",
    get: function get() {
      return this._defaults;
    }
  }]);

  return Event;
}();

exports.Event = Event;
exports.PluginProvider = PluginProvider;
exports.PluginStore = PluginStore;
exports.RendererPlugin = RendererPlugin;
exports.createPluginStore = createPluginStore;
exports.usePluginStore = usePluginStore;
//# sourceMappingURL=react-pluggable.cjs.development.js.map
